﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using UnivateProperties_API.Context;
using UnivateProperties_API.Model.Communication;

namespace UnivateProperties_API.Repository.Communication
{
    public class SMTPHostRepository : IRepository<SMTPHost>
    {
        private readonly DataContext _dbContext;

        public SMTPHostRepository(DataContext dbContext)
        {
            _dbContext = dbContext;
        }

        public List<SMTPHost> Get(Func<SMTPHost, bool> where)
        {
            return _dbContext.Hosts.Where(where).ToList();
        }

        public List<SMTPHost> GetAll()
        {
            return _dbContext.Hosts.ToList();
        }

        public SMTPHost GetDetailed(Func<SMTPHost, bool> first)
        {
            return _dbContext.Hosts.FirstOrDefault(first);
        }

        public List<SMTPHost> GetDetailedAll()
        {
            return GetAll();
        }

        public void Insert(SMTPHost item)
        {
            item.Id = NewId();
            _dbContext.Add(item);
            Save();
        }

        public void Insert(IEnumerable<SMTPHost> items)
        {
            int id = NewId();
            foreach (var item in items)
            {
                item.Id = id;
                _dbContext.Add(item);
                id += 1;
            }
            Save();
        }

        public void Remove(SMTPHost item)
        {
            var i = _dbContext.Hosts.Find(item);
            _dbContext.Hosts.Remove(i);
            Save();
        }

        public void Remove(IEnumerable<SMTPHost> items)
        {
            foreach (var item in items)
            {
                SMTPHost i = _dbContext.Hosts.Find(item);
                _dbContext.Hosts.Remove(i);
            }
            Save();
        }

        public void RemoveAtId(int item)
        {
            var i = _dbContext.Hosts.Find(item);
            _dbContext.Hosts.Remove(i);
            Save();
        }

        public void Update(SMTPHost item)
        {
            _dbContext.Entry(item).State = EntityState.Modified;
            Save();
        }

        public void Save()
        {
            _dbContext.SaveChanges();
        }

        public int NewId()
        {
            int id = 0;
            if (_dbContext.Hosts.Count() > 0)
            {
                id = _dbContext.Hosts.Max(x => x.Id);
            }
            id += 1;
            return id;
        }
    }
}
