﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using UnivateProperties_API.Context;
using UnivateProperties_API.Model.Properties;

namespace UnivateProperties_API.Repository.Properties
{
    public class UserDefinedFieldRepository : IRepository<UserDefinedField>
    {
        private readonly DataContext dBContext;

        public UserDefinedFieldRepository(DataContext _dBContext)
        {
            dBContext = _dBContext;
        }

        public List<UserDefinedField> Get(Func<UserDefinedField, bool> where)
        {
            return dBContext.UserDefinedFields.Where(where).OrderBy(x => x.Rank).ToList();
        }

        public List<UserDefinedField> GetAll()
        {
            return dBContext.UserDefinedFields.OrderBy(x => x.Rank).ToList();
        }

        public UserDefinedField GetDetailed(Func<UserDefinedField, bool> first)
        {
            return dBContext.UserDefinedFields.FirstOrDefault(first);
        }

        public List<UserDefinedField> GetDetailedAll()
        {
            return dBContext.UserDefinedFields.ToList();
        }

        public void Insert(UserDefinedField item)
        {
            dBContext.UserDefinedFields.Add(item);
            Save();
        }

        public void Insert(IEnumerable<UserDefinedField> items)
        {
            foreach (var item in items)
            {
                dBContext.UserDefinedFields.Add(item);
                Save();
            }
        }

        public void Remove(UserDefinedField item)
        {
            dBContext.UserDefinedFields.Remove(item);
            Save();
        }

        public void Remove(IEnumerable<UserDefinedField> items)
        {
            foreach (var item in items)
            {
                dBContext.UserDefinedFields.Add(item);
                Save();
            }
        }

        public void RemoveAtId(int item)
        {
            var userDefinedField = Get(x => x.Id == item).FirstOrDefault();
            if (userDefinedField != null)
            {
                dBContext.UserDefinedFields.Remove(userDefinedField);
                Save();
            }
        }

        public void Save()
        {
            dBContext.SaveChanges();
        }

        public void Update(UserDefinedField item)
        {
            dBContext.Entry(item).State = EntityState.Modified;
            Save();
        }

        public int NewId()
        {
            // Not sure if properties need it
            return 0;
        }
    }
}
