﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using UnivateProperties_API.Context;
using UnivateProperties_API.Model.Timeshare;

namespace UnivateProperties_API.Repository.Timeshare
{
    public class StatusRepository : IRepository<Status>
    {
        private readonly DataContext _dbContext;

        public StatusRepository(DataContext dbContext)
        {
            _dbContext = dbContext;
        }

        public List<Status> Get(Func<Status, bool> where)
        {
            return _dbContext.Status.Where(where).ToList();
        }

        public List<Status> GetAll()
        {
            return _dbContext.Status.ToList();
        }

        public Status GetDetailed(Func<Status, bool> first)
        {
            var item = _dbContext.Status.FirstOrDefault(first);
            return item;
        }

        public List<Status> GetDetailedAll()
        {
            return GetAll();
        }

        public void Insert(Status item)
        {
            item.Id = NewId();
            _dbContext.Add(item);
            Save();
        }

        public void Insert(IEnumerable<Status> items)
        {
            int id = NewId();
            foreach (var item in items)
            {
                item.Id = id;
                _dbContext.Add(item);
                id += 1;
            }
            Save();
        }

        public void Remove(Status item)
        {
            var i = _dbContext.Status.Find(item);
            _dbContext.Status.Remove(i);
            Save();
        }

        public void Remove(IEnumerable<Status> items)
        {
            foreach (var item in items)
            {
                var i = _dbContext.Status.Find(item);
                _dbContext.Status.Remove(i);
            }
            Save();
        }

        public void RemoveAtId(int item)
        {
            var i = _dbContext.Status.Find(item);
            _dbContext.Status.Remove(i);
            Save();
        }

        public void Save()
        {
            _dbContext.SaveChanges();
        }

        public void Update(Status item)
        {
            _dbContext.Entry(item).State = EntityState.Modified;
            Save();
        }

        public int NewId()
        {
            int id = 0;
            if (_dbContext.Status.Count() > 0)
            {
                id = _dbContext.Status.Max(x => x.Id);
            }
            id += 1;
            return id;
        }
    }
}
